// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Parabolic SAR according to Stocks and Commodities magazine V 11:11 (477-479).
    /// </summary>
    [Description("Parabolic SAR")]
    public class ParabolicSAR : Indicator
    {
        #region Variables
		private double acceleration			= 0.02;
		private double accelerationStep		= 0.02;
		private double accelerationMax		= 0.2;

		private bool   longPosition;
		private double xp					= 0;		// Extreme Price
		private double af					= 0;		// Acceleration factor
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.OrangeRed, PlotStyle.Dot, "Parabolic SAR"));
            CalculateOnBarClose		= true;	// Call 'OnBarUpdate' only as a bar closes
            Overlay					= true;	// Plots the indicator on top of price
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < 3) 
				return;
			else if (CurrentBar == 3)
			{
				// Determine initial position
				longPosition = High[0] > High[1] ? true : false;
				xp = MAX(longPosition ? High : Low, CurrentBar)[0];
				af = Acceleration;
				Value.Set(xp + (longPosition ? -1 : 1) * ((MAX(High, CurrentBar)[0] - MIN(Low, CurrentBar)[0]) * af));
				return;
			}

			// Reverse position
			if ((longPosition && Low[0] < Value[1]) || (!longPosition && High[0] > Value[1])) 
			{
				longPosition = !longPosition;
				Value.Set(xp);
				af = Acceleration;
				xp = longPosition ? High[0] : Low[0];
				return;
			}
			else if (longPosition && High[0] > xp)
			{
				xp = High[0];
				af = Math.Min(AccelerationMax, af + AccelerationStep);
			}
			else if (!longPosition && Low[0] < xp)
			{
				xp = Low[0];
				af = Math.Min(AccelerationMax, af + AccelerationStep);
			}

			// Additional rule. SAR for today can not be placed inside the bar of day - 1 or day - 2.
			double todaySAR = Value[1] + af * (xp - Value[1]);
			todaySAR = longPosition ? Math.Min(Math.Min(todaySAR, Low[1]),  Low[2]) : 
									  Math.Max(Math.Max(todaySAR, High[1]), High[2]);
			
			Value.Set(todaySAR);
		}

        #region Properties
        /// <summary>
        /// The initial acceleration factor
        /// </summary>
        [Description("The initial acceleration factor")]
        [Category("Parameters")]
        public double Acceleration
        {
            get { return acceleration; }
            set { acceleration = Math.Max(0.02, value); }
        }

        /// <summary>
        /// The acceleration step factor
        /// </summary>
        [Description("The acceleration step factor")]
        [Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Acceleration step")]
        public double AccelerationStep
        {
            get { return accelerationStep; }
            set { accelerationStep = Math.Max(0.02, value); }
        }

		/// <summary>
		/// The maximum acceleration
		/// </summary>
		[Description("The maximum acceleration")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Acceleration max")]
		public double AccelerationMax
		{
			get { return accelerationMax; }
			set { accelerationMax = Math.Max(0.02, value); }
		}
		#endregion

    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ParabolicSAR[] cacheParabolicSAR = null;
        private static ParabolicSAR checkParabolicSAR = new ParabolicSAR();

        /// <summary>
        /// Parabolic SAR
        /// </summary>
        /// <returns></returns>
        public ParabolicSAR ParabolicSAR(double acceleration, double accelerationMax, double accelerationStep)
        {
            return ParabolicSAR(Input, acceleration, accelerationMax, accelerationStep);
        }

        /// <summary>
        /// Parabolic SAR
        /// </summary>
        /// <returns></returns>
        public ParabolicSAR ParabolicSAR(Data.IDataSeries input, double acceleration, double accelerationMax, double accelerationStep)
        {
            checkParabolicSAR.Acceleration = acceleration;
            acceleration = checkParabolicSAR.Acceleration;
            checkParabolicSAR.AccelerationMax = accelerationMax;
            accelerationMax = checkParabolicSAR.AccelerationMax;
            checkParabolicSAR.AccelerationStep = accelerationStep;
            accelerationStep = checkParabolicSAR.AccelerationStep;

            if (cacheParabolicSAR != null)
                for (int idx = 0; idx < cacheParabolicSAR.Length; idx++)
                    if (Math.Abs(cacheParabolicSAR[idx].Acceleration - acceleration) <= double.Epsilon && Math.Abs(cacheParabolicSAR[idx].AccelerationMax - accelerationMax) <= double.Epsilon && Math.Abs(cacheParabolicSAR[idx].AccelerationStep - accelerationStep) <= double.Epsilon && cacheParabolicSAR[idx].EqualsInput(input))
                        return cacheParabolicSAR[idx];

            ParabolicSAR indicator = new ParabolicSAR();
			indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Acceleration = acceleration;
            indicator.AccelerationMax = accelerationMax;
            indicator.AccelerationStep = accelerationStep;
            indicator.SetUp();

            ParabolicSAR[] tmp = new ParabolicSAR[cacheParabolicSAR == null ? 1 : cacheParabolicSAR.Length + 1];
            if (cacheParabolicSAR != null)
                cacheParabolicSAR.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheParabolicSAR = tmp;
            Indicators.Add(indicator);

            return indicator;
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Parabolic SAR
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ParabolicSAR ParabolicSAR(double acceleration, double accelerationMax, double accelerationStep)
        {
            return _indicator.ParabolicSAR(Input, acceleration, accelerationMax, accelerationStep);
        }

        /// <summary>
        /// Parabolic SAR
        /// </summary>
        /// <returns></returns>
        public Indicator.ParabolicSAR ParabolicSAR(Data.IDataSeries input, double acceleration, double accelerationMax, double accelerationStep)
        {
            return _indicator.ParabolicSAR(input, acceleration, accelerationMax, accelerationStep);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Parabolic SAR
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ParabolicSAR ParabolicSAR(double acceleration, double accelerationMax, double accelerationStep)
        {
            return _indicator.ParabolicSAR(Input, acceleration, accelerationMax, accelerationStep);
        }

        /// <summary>
        /// Parabolic SAR
        /// </summary>
        /// <returns></returns>
        public Indicator.ParabolicSAR ParabolicSAR(Data.IDataSeries input, double acceleration, double accelerationMax, double accelerationStep)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ParabolicSAR(input, acceleration, accelerationMax, accelerationStep);
        }
    }
}
#endregion
