// 
// Copyright (C) 2007, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Sends an alert to the alert window when a price level has been reached.
	/// </summary>
	[Description("Sends an alert to the alert window when a price level has been reached.")]
	public class PriceAlert : Indicator
	{
		#region Variables
		private double					price						= 0;
		private bool					showTriggerLine				= true;
		private bool					triggered					= false;
		private bool					triggerOnGreaterThan		= false;
		private bool					triggerSet					= false;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Navy, PlotStyle.Line, "Trigger line"));

			ChartOnly			= true;
			AutoScale			= false;
			CalculateOnBarClose = false;
			DisplayInDataBox	= false;
			Overlay				= true;
			PriceTypeSupported	= false;
		}
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (ShowTriggerLine)
				Values[0].Set(Price);

			if (Historical || triggered)
				return;
			else if (!triggerSet)
			{
				triggerOnGreaterThan = Close[0] >= price - (TickSize * 0.5) ? false : true;
				triggerSet = true;
			}

			if ((triggerOnGreaterThan && Close[0] >= Price - (TickSize * 0.5)) || (!triggerOnGreaterThan && Close[0] <= Price + (TickSize * 0.5)))
			{
				triggered = true;
				Alert(DateTime.Now.Millisecond.ToString(), NinjaTrader.Cbi.Priority.Medium, "Price level '" + Price + "' hit!", Cbi.Core.InstallDir + @"\sounds\Alert4.wav", 0, Color.Yellow, Color.Black);
			}
		}


		#region Properties
		/// <summary>
		/// </summary>
		[Description("The price alert level.")]
		[Category("Parameters")]
		public double Price
		{
			get { return price; }
			set { price = Math.Max(0, value); }
		}

		[Browsable(true)]
		[Gui.Design.DisplayNameAttribute("Show trigger line")]
		[Category("Parameters")]
		public bool ShowTriggerLine
		{
			get { return showTriggerLine; }
			set { showTriggerLine = value; }
		}
		#endregion
	}
}



#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private PriceAlert[] cachePriceAlert = null;
        private static PriceAlert checkPriceAlert = new PriceAlert();

        /// <summary>
        /// Sends an alert to the alert window when a price level has been reached.
        /// </summary>
        /// <returns></returns>
        public PriceAlert PriceAlert(double price, bool showTriggerLine)
        {
            return PriceAlert(Input, price, showTriggerLine);
        }

        /// <summary>
        /// Sends an alert to the alert window when a price level has been reached.
        /// </summary>
        /// <returns></returns>
        public PriceAlert PriceAlert(Data.IDataSeries input, double price, bool showTriggerLine)
        {
            checkPriceAlert.Price = price;
            price = checkPriceAlert.Price;
            checkPriceAlert.ShowTriggerLine = showTriggerLine;
            showTriggerLine = checkPriceAlert.ShowTriggerLine;

            if (cachePriceAlert != null)
                for (int idx = 0; idx < cachePriceAlert.Length; idx++)
                    if (Math.Abs(cachePriceAlert[idx].Price - price) <= double.Epsilon && cachePriceAlert[idx].ShowTriggerLine == showTriggerLine && cachePriceAlert[idx].EqualsInput(input))
                        return cachePriceAlert[idx];

            PriceAlert indicator = new PriceAlert();
			indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Price = price;
            indicator.ShowTriggerLine = showTriggerLine;
            indicator.SetUp();

            PriceAlert[] tmp = new PriceAlert[cachePriceAlert == null ? 1 : cachePriceAlert.Length + 1];
            if (cachePriceAlert != null)
                cachePriceAlert.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cachePriceAlert = tmp;
            Indicators.Add(indicator);

            return indicator;
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Sends an alert to the alert window when a price level has been reached.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PriceAlert PriceAlert(double price, bool showTriggerLine)
        {
            return _indicator.PriceAlert(Input, price, showTriggerLine);
        }

        /// <summary>
        /// Sends an alert to the alert window when a price level has been reached.
        /// </summary>
        /// <returns></returns>
        public Indicator.PriceAlert PriceAlert(Data.IDataSeries input, double price, bool showTriggerLine)
        {
            return _indicator.PriceAlert(input, price, showTriggerLine);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Sends an alert to the alert window when a price level has been reached.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PriceAlert PriceAlert(double price, bool showTriggerLine)
        {
            return _indicator.PriceAlert(Input, price, showTriggerLine);
        }

        /// <summary>
        /// Sends an alert to the alert window when a price level has been reached.
        /// </summary>
        /// <returns></returns>
        public Indicator.PriceAlert PriceAlert(Data.IDataSeries input, double price, bool showTriggerLine)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.PriceAlert(input, price, showTriggerLine);
        }
    }
}
#endregion
