// MACDUpDown v1.1
// Copyright (C) 2008, Wessel de Roode
// Based on the original MACD Script of Ninjatrader. Added the bar coloring
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The MACDUpDown (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
	/// </summary>
	[Description("The MACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices. The histogram is green if the MACD ticks up and turns red if the histogram ticks down.")]
	public class MACDUpDown : Indicator
	{
		#region Variables
		private int					fast	= 12;
		private int					slow	= 26;
		private int					smooth	= 9;
		private	DataSeries		fastEma;
		private	DataSeries		slowEma;
		private	DataSeries		diffArr;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Green, "Macd"));
			Add(new Plot(Color.DarkViolet, "Avg"));
			Add(new Plot(new Pen(Color.Green, 3), PlotStyle.Bar, "HistTickUp"));
			Add(new Plot(new Pen(Color.Red, 3), PlotStyle.Bar, "HistTickDown"));
			
			Add(new Line(Color.DarkGray, 0, "Zero line"));

			fastEma	= new DataSeries(this);
			slowEma	= new DataSeries(this);
			diffArr = new DataSeries(this);
		}

		/// <summary>
		/// Calculates the indicator value(s) at the current index.
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (CurrentBar == 0)
			{
				fastEma.Set(Input[0]);
				slowEma.Set(Input[0]);
				Value.Set(0);
				Avg.Set(0);
				diffArr.Set(Input[0]);
				DiffUp.Set(0);
				DiffDown.Set(0);
			}
			else
			{
				fastEma.Set((2.0 / (1 + Fast)) * Input[0] + (1 - (2.0 / (1 + Fast))) * fastEma[1]);
				slowEma.Set((2.0 / (1 + Slow)) * Input[0] + (1 - (2.0 / (1 + Slow))) * slowEma[1]);

				double macd		= fastEma[0] - slowEma[0];
				double macdAvg	= (2.0 / (1 + Smooth)) * macd + (1 - (2.0 / (1 + Smooth))) * Avg[1];
				
				diffArr.Set(macd - macdAvg);
				
				Value.Set(macd);
				Avg.Set(macdAvg);
				if( diffArr.Get(CurrentBar)>diffArr.Get(CurrentBar - 1)){
					DiffUp.Set(macd - macdAvg);
				} else {
					DiffDown.Set(macd - macdAvg);
				}				
			}
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Avg
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Default
		{
			get { return Values[0]; }
		}
		

        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries DiffUp
		{
			get { return Values[2]; }
		}
		
		        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries DiffDown
		{
			get { return Values[3]; }
		}
		
		
		/// <summary>
		/// </summary>
		[Description("Number of bars for fast EMA")]
		[Category("Parameters")]
		public int Fast
		{
			get { return fast; }
			set { fast = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for slow EMA")]
		[Category("Parameters")]
		public int Slow
		{
			get { return slow; }
			set { slow = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private MACDUpDown[] cacheMACDUpDown = null;

        private static MACDUpDown checkMACDUpDown = new MACDUpDown();

        /// <summary>
        /// The MACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices. The histogram is green if the MACD ticks up and turns red if the histogram ticks down.
        /// </summary>
        /// <returns></returns>
        public MACDUpDown MACDUpDown(int fast, int slow, int smooth)
        {
            return MACDUpDown(Input, fast, slow, smooth);
        }

        /// <summary>
        /// The MACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices. The histogram is green if the MACD ticks up and turns red if the histogram ticks down.
        /// </summary>
        /// <returns></returns>
        public MACDUpDown MACDUpDown(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            checkMACDUpDown.Fast = fast;
            fast = checkMACDUpDown.Fast;
            checkMACDUpDown.Slow = slow;
            slow = checkMACDUpDown.Slow;
            checkMACDUpDown.Smooth = smooth;
            smooth = checkMACDUpDown.Smooth;

            if (cacheMACDUpDown != null)
                for (int idx = 0; idx < cacheMACDUpDown.Length; idx++)
                    if (cacheMACDUpDown[idx].Fast == fast && cacheMACDUpDown[idx].Slow == slow && cacheMACDUpDown[idx].Smooth == smooth && cacheMACDUpDown[idx].EqualsInput(input))
                        return cacheMACDUpDown[idx];

            MACDUpDown indicator = new MACDUpDown();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Fast = fast;
            indicator.Slow = slow;
            indicator.Smooth = smooth;
            indicator.SetUp();

            MACDUpDown[] tmp = new MACDUpDown[cacheMACDUpDown == null ? 1 : cacheMACDUpDown.Length + 1];
            if (cacheMACDUpDown != null)
                cacheMACDUpDown.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheMACDUpDown = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The MACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices. The histogram is green if the MACD ticks up and turns red if the histogram ticks down.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MACDUpDown MACDUpDown(int fast, int slow, int smooth)
        {
            return _indicator.MACDUpDown(Input, fast, slow, smooth);
        }

        /// <summary>
        /// The MACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices. The histogram is green if the MACD ticks up and turns red if the histogram ticks down.
        /// </summary>
        /// <returns></returns>
        public Indicator.MACDUpDown MACDUpDown(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            return _indicator.MACDUpDown(input, fast, slow, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The MACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices. The histogram is green if the MACD ticks up and turns red if the histogram ticks down.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MACDUpDown MACDUpDown(int fast, int slow, int smooth)
        {
            return _indicator.MACDUpDown(Input, fast, slow, smooth);
        }

        /// <summary>
        /// The MACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices. The histogram is green if the MACD ticks up and turns red if the histogram ticks down.
        /// </summary>
        /// <returns></returns>
        public Indicator.MACDUpDown MACDUpDown(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.MACDUpDown(input, fast, slow, smooth);
        }

    }
}
#endregion
