//
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

// Indicator: RSI Overlay
// Author: cvax
// Version: 0.1.0

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
	[Description("The RSI (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.")]
	[Gui.Design.DisplayName("RSI (Overlay on Price)")]
	public class RSIOverlay : Indicator
	{
		#region Variables
		private DataSeries					avgUp;
		private DataSeries					avgDown;
		private DataSeries					down;
		private DataSeries					up;
		private int							period	= 14;
		private int							smooth	= 3;
		private int							upperline = 70;
		private int							lowerline = 30;
		private Color						uppercolor = Color.YellowGreen;
		private Color						lowercolor = Color.DarkViolet;
		private bool						drawscale = true;
		
		private StringFormat	stringFormat	= new StringFormat();
		private SolidBrush		textBrush		= new SolidBrush(Color.Black);
		private System.Drawing.Font textFont	= new Font("Arial", 8);
		#endregion

		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Green,2), "RSI"));
			Add(new Plot(new Pen(Color.Orange,2), "Avg"));
			avgUp				= new DataSeries(this);
			avgDown				= new DataSeries(this);
			down				= new DataSeries(this);
			up					= new DataSeries(this);

			PriceTypeSupported	= true;			
			AutoScale 			= false;
			Overlay				= true;
			CalculateOnBarClose = false;
			PaintPriceMarkers	= false;
		}

		protected override void OnBarUpdate()
		{
			if(AutoScale)
				AutoScale = false;
			if (CurrentBar == 0)
			{
				down.Set(0);
				up.Set(0);
				return;
			}

			down.Set(Math.Max(Input[1] - Input[0], 0));
			up.Set(Math.Max(Input[0] - Input[1], 0));

			if ((CurrentBar + 1) < Period) 
			{
				if ((CurrentBar + 1) == (Period - 1))
					Avg.Set(50);
				return;
			}

			if ((CurrentBar + 1) == Period) 
			{
				// First averages 
				avgDown.Set(SMA(down, Period)[0]);
				avgUp.Set(SMA(up, Period)[0]);
			}  
			else 
			{
				// Rest of averages are smoothed
				avgDown.Set((avgDown[1] * (Period - 1) + down[0]) / Period);
				avgUp.Set((avgUp[1] * (Period - 1) + up[0]) / Period);
			}

			double rs	  = avgUp[0] / (avgDown[0] == 0 ? 1 : avgDown[0]);
			double rsi	  = 100 - (100 / (1 + rs));
			double rsiAvg = (2.0 / (1 + Smooth)) * rsi + (1 - (2.0 / (1 + Smooth))) * Avg[1];

			Avg.Set(rsiAvg);
			Value.Set(rsi);
		}

		#region Properties
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Avg
		{
			get { return Values[1]; }
		}

		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Default
		{
			get { return Values[0]; }
		}
		
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(2, value); }
		}

		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}
		
		[Description("Draw Scale Guidelines")]
		[Category("Lines")]
		[Gui.Design.DisplayName("Draw Scale")]
		public bool DrawScale
		{
			get { return drawscale; }
			set { drawscale = value; }
		}
		
		[Description("Value for Lower Line")]
		[Category("Lines")]
		public int Lower
		{
			get { return lowerline; }
			set { lowerline = Math.Min(Math.Max(0,value),100); }
		}
		
		[Description("Value for Upper Line")]
		[Category("Lines")]
		public int Upper
		{
			get { return upperline; }
			set { upperline = Math.Min(Math.Max(0,value),100); }
		}
		
		[Description("Lower Color")]
		[Category("Lines")]
		public Color LowerColor
		{
			get { return lowercolor; }
			set { lowercolor = value; }
		}
		
		[Description("Upper Color")]
		[Category("Lines")]
		public Color UpperColor
		{
			get { return uppercolor; }
			set { uppercolor = value; }
		}
		#endregion
		
		#region Miscellaneous
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			//base.Plot(graphics, bounds, min, max);
			//Lines
			if(drawscale)
			{
				graphics.DrawLine(new Pen(uppercolor,2),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((double)upperline/100))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((double)upperline/100))));
				graphics.DrawString(upperline.ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((double)upperline/100))),stringFormat);
				graphics.DrawLine(new Pen(Color.DarkGray,1),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((((double)upperline/100)-0.50)/2+0.50))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((((double)upperline/100)-0.50)/2+0.50))));
				graphics.DrawString(((upperline-50)/2+50).ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((((double)upperline/100)-0.50)/2+0.50))),stringFormat);
				graphics.DrawLine(new Pen(Color.DarkGray,1),bounds.X,(float)((bounds.Bottom-bounds.Y)*0.50),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*0.50));
				graphics.DrawString("50", textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-0.50)),stringFormat);
				graphics.DrawLine(new Pen(Color.DarkGray,1),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((0.50-((double)lowerline/100))/2+((double)lowerline/100)))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((0.50-((double)lowerline/100))/2+((double)lowerline/100)))));
				graphics.DrawString(((50-lowerline)/2+lowerline).ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((0.50-((double)lowerline/100))/2+((double)lowerline/100)))),stringFormat);
				graphics.DrawLine(new Pen(lowercolor,2),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((double)lowerline/100))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((double)lowerline/100))));
				graphics.DrawString(lowerline.ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((double)lowerline/100))),stringFormat);
			}

			//Stochastic
			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			SmoothingMode oldSmoothingMode = graphics.SmoothingMode;
			GraphicsPath rsipath = new GraphicsPath();
			GraphicsPath avgpath = new GraphicsPath();
			for(int seriesCount = 0; seriesCount < 2; seriesCount++)
			{
				int lastX = -1;
				int lastY = -1;
				DataSeries series = (DataSeries) Values[seriesCount];
				double seriesvalue = 0;
				Gui.Chart.Plot plot = Plots[seriesCount];
				for(int count = 0; count < ChartControl.BarsPainted; count++)
				{
					int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
						if (idx < 0 || idx >= Input.Count || (!ChartControl.ShowBarsRequired && idx < BarsRequired))
							continue;
						
						seriesvalue	= series.Get(idx)/100;
						if (seriesvalue == 0)
							continue;
	
						int	x = (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2 - (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + count * ChartControl.BarSpace) + 1;
						int y = (int) ((bounds.Bottom-bounds.Y)*(1-seriesvalue));
	
						if (lastX >= 0) 
						{
							if(seriesCount == 0)
								rsipath.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
							else
								avgpath.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
	
						lastX	= x;
						lastY	= y;
				}
				rsipath.Reverse();
				avgpath.Reverse();
			}
			graphics.SmoothingMode = SmoothingMode.AntiAlias;
			graphics.DrawPath(Plots[0].Pen, rsipath);
			graphics.DrawPath(Plots[1].Pen, avgpath);
			graphics.SmoothingMode = oldSmoothingMode;
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RSIOverlay[] cacheRSIOverlay = null;

        private static RSIOverlay checkRSIOverlay = new RSIOverlay();

        /// <summary>
        /// The RSI (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public RSIOverlay RSIOverlay(int period, int smooth)
        {
            return RSIOverlay(Input, period, smooth);
        }

        /// <summary>
        /// The RSI (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public RSIOverlay RSIOverlay(Data.IDataSeries input, int period, int smooth)
        {
            checkRSIOverlay.Period = period;
            period = checkRSIOverlay.Period;
            checkRSIOverlay.Smooth = smooth;
            smooth = checkRSIOverlay.Smooth;

            if (cacheRSIOverlay != null)
                for (int idx = 0; idx < cacheRSIOverlay.Length; idx++)
                    if (cacheRSIOverlay[idx].Period == period && cacheRSIOverlay[idx].Smooth == smooth && cacheRSIOverlay[idx].EqualsInput(input))
                        return cacheRSIOverlay[idx];

            RSIOverlay indicator = new RSIOverlay();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;
            indicator.Smooth = smooth;

            RSIOverlay[] tmp = new RSIOverlay[cacheRSIOverlay == null ? 1 : cacheRSIOverlay.Length + 1];
            if (cacheRSIOverlay != null)
                cacheRSIOverlay.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRSIOverlay = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The RSI (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RSIOverlay RSIOverlay(int period, int smooth)
        {
            return _indicator.RSIOverlay(Input, period, smooth);
        }

        /// <summary>
        /// The RSI (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public Indicator.RSIOverlay RSIOverlay(Data.IDataSeries input, int period, int smooth)
        {
            return _indicator.RSIOverlay(input, period, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The RSI (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RSIOverlay RSIOverlay(int period, int smooth)
        {
            return _indicator.RSIOverlay(Input, period, smooth);
        }

        /// <summary>
        /// The RSI (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public Indicator.RSIOverlay RSIOverlay(Data.IDataSeries input, int period, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RSIOverlay(input, period, smooth);
        }

    }
}
#endregion
