//
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

// Indicator: Stochastics Overlay
// Author: cvax
// Version: 0.1.2

// Changelog:
// v0.1.2 - Fixed error in calculations for D and K lines
// v0.1.1 - Added ability to customize scale
// v0.1.0 - Public Release

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
	[Description("The Stochastic Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.")]
	[Gui.Design.DisplayName("Stochastic Oscillator (Overlay on Price)")]
	public class StochasticsOverlay : Indicator
	{
		#region Variables
		private int					periodD	= 7;
		private int					periodK	= 14;
		private int					smooth	= 3;
		private DataSeries			den;
		private DataSeries			nom;
		private bool				drawscale = true;
		
		private int							upperline = 80;
		private int							lowerline = 20;
		private Color						uppercolor = Color.YellowGreen;
		private Color						lowercolor = Color.DarkViolet;
		
		private StringFormat	stringFormat	= new StringFormat();
		private SolidBrush		textBrush		= new SolidBrush(Color.Black);
		private System.Drawing.Font textFont	= new Font("Arial", 8);
		#endregion

		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Green,2), "Stoc. D"));
			Add(new Plot(new Pen(Color.Orange,2), "Stoc. K"));
			den			= new DataSeries(this);
			nom			= new DataSeries(this);
			
			AutoScale 			= false;
			Overlay				= true;
			CalculateOnBarClose = false;
			PaintPriceMarkers	= false;
		}

		protected override void OnBarUpdate()
		{
			if(AutoScale)
				AutoScale = false;
			den.Set(MAX(High, PeriodK)[0] - MIN(Low, PeriodK)[0]);
			nom.Set(Close[0] - MIN(Low, PeriodK)[0]);
			
			K.Set(100 * SUM(nom, Smooth)[0] / (SUM(den, Smooth)[0] == 0 ? 1.0 : SUM(den, Smooth)[0]));
			D.Set(SMA(K, PeriodD)[0]);
		}

		#region Properties
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries D
		{
			get { return Values[0]; }
		}

		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries K
		{
			get { return Values[1]; }
		}
		
		[Description("Numbers of bars used for calculating the D values")]
		[Category("Parameters")]
		public int PeriodD
		{
			get { return periodD; }
			set { periodD = Math.Max(1, value); }
		}

		[Description("Numbers of bars used for calculating the K values")]
		[Category("Parameters")]
		public int PeriodK
		{
			get { return periodK; }
			set { periodK = Math.Max(1, value); }
		}

		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}
		
		[Description("Draw Scale Guidelines")]
		[Category("Lines")]
		[Gui.Design.DisplayName("Draw Scale")]
		public bool DrawScale
		{
			get { return drawscale; }
			set { drawscale = value; }
		}
		
		[Description("Value for Lower Line")]
		[Category("Lines")]
		public int Lower
		{
			get { return lowerline; }
			set { lowerline = Math.Min(Math.Max(0,value),100); }
		}
		
		[Description("Value for Upper Line")]
		[Category("Lines")]
		public int Upper
		{
			get { return upperline; }
			set { upperline = Math.Min(Math.Max(0,value),100); }
		}
		
		[Description("Lower Color")]
		[Category("Lines")]
		public Color LowerColor
		{
			get { return lowercolor; }
			set { lowercolor = value; }
		}
		
		[Description("Upper Color")]
		[Category("Lines")]
		public Color UpperColor
		{
			get { return uppercolor; }
			set { uppercolor = value; }
		}
		#endregion
		
		#region Miscellaneous
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			//base.Plot(graphics, bounds, min, max);
			//Lines
			if(drawscale)
			{
				graphics.DrawLine(new Pen(uppercolor,2),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((double)upperline/100))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((double)upperline/100))));
				graphics.DrawString(upperline.ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((double)upperline/100))),stringFormat);
				graphics.DrawLine(new Pen(Color.DarkGray,1),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((((double)upperline/100)-0.50)/2+0.50))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((((double)upperline/100)-0.50)/2+0.50))));
				graphics.DrawString(((upperline-50)/2+50).ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((((double)upperline/100)-0.50)/2+0.50))),stringFormat);
				graphics.DrawLine(new Pen(Color.DarkGray,1),bounds.X,(float)((bounds.Bottom-bounds.Y)*0.50),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*0.50));
				graphics.DrawString("50", textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-0.50)),stringFormat);
				graphics.DrawLine(new Pen(Color.DarkGray,1),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((0.50-((double)lowerline/100))/2+((double)lowerline/100)))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((0.50-((double)lowerline/100))/2+((double)lowerline/100)))));
				graphics.DrawString(((50-lowerline)/2+lowerline).ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((0.50-((double)lowerline/100))/2+((double)lowerline/100)))),stringFormat);
				graphics.DrawLine(new Pen(lowercolor,2),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((double)lowerline/100))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((double)lowerline/100))));
				graphics.DrawString(lowerline.ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((double)lowerline/100))),stringFormat);
			}

			//Stochastic
			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			SmoothingMode oldSmoothingMode = graphics.SmoothingMode;
			GraphicsPath dpath = new GraphicsPath();
			GraphicsPath kpath = new GraphicsPath();
			for(int seriesCount = 0; seriesCount < 2; seriesCount++)
			{
				int lastX = -1;
				int lastY = -1;
				DataSeries series = (DataSeries) Values[seriesCount];
				double seriesvalue = 0;
				Gui.Chart.Plot plot = Plots[seriesCount];
				for(int count = 0; count < ChartControl.BarsPainted; count++)
				{
					int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
						if (idx < 0 || idx >= Input.Count || (!ChartControl.ShowBarsRequired && idx < BarsRequired))
							continue;
						
						seriesvalue	= series.Get(idx)/100;
						if (seriesvalue == 0)
							continue;
	
						int	x = (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2 - (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + count * ChartControl.BarSpace) + 1;
						int y = (int) ((bounds.Bottom-bounds.Y)*(1-seriesvalue));
	
						if (lastX >= 0) 
						{
							if(seriesCount == 0)
								dpath.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
							else
								kpath.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
	
						lastX	= x;
						lastY	= y;
				}
				dpath.Reverse();
				kpath.Reverse();
			}
			graphics.SmoothingMode = SmoothingMode.AntiAlias;
			graphics.DrawPath(Plots[0].Pen, dpath);
			graphics.DrawPath(Plots[1].Pen, kpath);
			graphics.SmoothingMode = oldSmoothingMode;
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private StochasticsOverlay[] cacheStochasticsOverlay = null;

        private static StochasticsOverlay checkStochasticsOverlay = new StochasticsOverlay();

        /// <summary>
        /// The Stochastic Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        public StochasticsOverlay StochasticsOverlay(int periodD, int periodK, int smooth)
        {
            return StochasticsOverlay(Input, periodD, periodK, smooth);
        }

        /// <summary>
        /// The Stochastic Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        public StochasticsOverlay StochasticsOverlay(Data.IDataSeries input, int periodD, int periodK, int smooth)
        {
            checkStochasticsOverlay.PeriodD = periodD;
            periodD = checkStochasticsOverlay.PeriodD;
            checkStochasticsOverlay.PeriodK = periodK;
            periodK = checkStochasticsOverlay.PeriodK;
            checkStochasticsOverlay.Smooth = smooth;
            smooth = checkStochasticsOverlay.Smooth;

            if (cacheStochasticsOverlay != null)
                for (int idx = 0; idx < cacheStochasticsOverlay.Length; idx++)
                    if (cacheStochasticsOverlay[idx].PeriodD == periodD && cacheStochasticsOverlay[idx].PeriodK == periodK && cacheStochasticsOverlay[idx].Smooth == smooth && cacheStochasticsOverlay[idx].EqualsInput(input))
                        return cacheStochasticsOverlay[idx];

            StochasticsOverlay indicator = new StochasticsOverlay();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.PeriodD = periodD;
            indicator.PeriodK = periodK;
            indicator.Smooth = smooth;
            indicator.SetUp();

            StochasticsOverlay[] tmp = new StochasticsOverlay[cacheStochasticsOverlay == null ? 1 : cacheStochasticsOverlay.Length + 1];
            if (cacheStochasticsOverlay != null)
                cacheStochasticsOverlay.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheStochasticsOverlay = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The Stochastic Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.StochasticsOverlay StochasticsOverlay(int periodD, int periodK, int smooth)
        {
            return _indicator.StochasticsOverlay(Input, periodD, periodK, smooth);
        }

        /// <summary>
        /// The Stochastic Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        public Indicator.StochasticsOverlay StochasticsOverlay(Data.IDataSeries input, int periodD, int periodK, int smooth)
        {
            return _indicator.StochasticsOverlay(input, periodD, periodK, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The Stochastic Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.StochasticsOverlay StochasticsOverlay(int periodD, int periodK, int smooth)
        {
            return _indicator.StochasticsOverlay(Input, periodD, periodK, smooth);
        }

        /// <summary>
        /// The Stochastic Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        public Indicator.StochasticsOverlay StochasticsOverlay(Data.IDataSeries input, int periodD, int periodK, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.StochasticsOverlay(input, periodD, periodK, smooth);
        }

    }
}
#endregion
